package yrpg.bamp.basedata.webapi;

import kd.bos.bill.IBillWebApiPlugin;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.operate.result.OperateErrorInfo;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.entity.validate.ValidateResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.bamp.basedata.webapi.pojo.RelationDTO;


import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@ApiController(value = "relation", desc = "人区客品关系")
@ApiMapping("/relation")
public class NewRelationSaveApiPlugin implements IBillWebApiPlugin {

    private static final DynamicObject yzj = BusinessDataServiceHelper.loadSingle(100000L, "bos_org");

    @ApiPostMapping(value = "/saveRelationInformation", desc = "主数据人区客品关系")
    public CustomApiResult<Object> saveRelationInformation(
            @ApiParam(value = "编码", required = true, message = "人区客品编码不能为空") String number,
            @ApiParam(value = "业务员编码", required = true, message = "业务员编码不能为空") String salesman,
            @ApiParam(value = "区域编码", required = true, message = "区域编码不能为空") String office,
            @ApiParam(value = "客户编码", required = true, message = "客户编码不能为空") String customer,
            @ApiParam(value = "品种编码") String goods,
            @ApiParam(value = "开始日期", required = true, message = "开始日期不能为空") Date start_date,
            @ApiParam(value = "结束日期", required = true, message = "结束日期不能为空") Date end_date,
            @ApiParam(value = "台账人编码", required = true, message = "台账人编码不能为空") String account_man,
            @ApiParam(value = "操作类型", required = true, message = "操作类型不能为空") String operationType
    ) {
        DynamicObject relation = BusinessDataServiceHelper.loadSingle("yrpg_mkt_sales_area_manag",
                QFilter.of("yrpg_master_data_number = ?", number).toArray());
        if (relation != null) {
            relation.set("yrpg_end_date", end_date);
        } else {
            DynamicObject org = BusinessDataServiceHelper.loadSingle("bos_adminorg",
                    QFilter.of("yrpg_mainid = ?", office).toArray());
            DynamicObject sales = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                    QFilter.of("number = ?", salesman).toArray());
            DynamicObject account = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                    QFilter.of("number = ?", account_man).toArray());
            DynamicObject mktCustomer = BusinessDataServiceHelper.loadSingle("yrpg_bd_endcustomer",
                    QFilter.of("number = ?", customer).toArray());
            DynamicObject mktGoods = goods == null || goods.isEmpty() ? null :
                    BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                            QFilter.of("number = ?", goods).toArray());
            DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
            if (org == null) {
                return CustomApiResult.fail("", "未查询到区域编码对应的区域,区域编码:{" + office + "}");
            }
            if (sales == null) {
                return CustomApiResult.fail("", "未查询到业务员编码对应的业务员,业务员编码:{" + salesman + "}");
            }
            if (account == null) {
                return CustomApiResult.fail("", "未查询到台账人编码对应的台账人,台账人编码:{" + account_man + "}");
            }
            if (mktCustomer == null) {
                return CustomApiResult.fail("", "未查询到客户编码对应的客户,客户编码:{" + customer + "}");
            }
            if (goods != null && !goods.isEmpty() && mktGoods == null) {
                return CustomApiResult.fail("", "未查询到品种编码对应的区域,品种编码:{" + goods + "}");
            }
            DynamicObject mktOffice = BusinessDataServiceHelper.loadSingle("rm_mkt_office",
                    QFilter.of("number = ?", office).toArray());
            if (mktOffice == null) {
                mktOffice = createOfficeInfo(org, user);
            }
            relation = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_sales_area_manag");
            relation.set("yrpg_master_data_number", number);
            relation.set("yrpg_customer_dimension", "1");
            relation.set("yrpg_goods_dimension", "1");
            relation.set("yrpg_org", org);
            relation.set("yrpg_office_id", mktOffice);
            relation.set("yrpg_salesman_id", sales);
            relation.set("yrpg_account_man_id", account);
            relation.set("yrpg_customer_id", mktCustomer);
            relation.set("yrpg_goods_id", mktGoods);
            relation.set("yrpg_start_date", start_date);
            relation.set("yrpg_end_date", end_date);
            relation.set("creator", user);
            relation.set("auditor", user);
            relation.set("auditdate", new Date());
            relation.set("billstatus", "C");
        }
        OperationResult result = OperationServiceHelper.executeOperate("save", "yrpg_mkt_sales_area_manag", new DynamicObject[]{relation});
        if (result.isSuccess()) {
            return CustomApiResult.success("success");
       }
       else {
            List<ValidateResult> validateErrors = result.getValidateResult().getValidateErrors();
            if (!validateErrors.isEmpty()) {
                List<OperateErrorInfo> allErrorInfo = validateErrors.get(0).getAllErrorInfo();
                if (!allErrorInfo.isEmpty()) {
                    StringBuilder message = new StringBuilder();
                    for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                        message.append(operateErrorInfo.getMessage());
                    }
                    return CustomApiResult.fail("", message.toString());
                } else {
                    return CustomApiResult.fail("", result.getMessage());
                }
            } else {
                return CustomApiResult.fail("", result.getMessage());
            }
        }
    }

    @ApiPostMapping(value = "/batchSaveRelationInformation", desc = "主数据人区客品关系")
    public CustomApiResult<Object> batchSaveRelationInformation(
            @ApiParam(value = "人区客品数据", required = true, message = "人区客品数据不能为空") List<RelationDTO> relationList
    ) {
        if (relationList.size() > 1000) {
            return CustomApiResult.fail("", "请一次性处理1000条以下的数据");
        }
        ArrayList<DynamicObject> list = new ArrayList<>();
        DynamicObject user = BusinessDataServiceHelper.loadSingle(RequestContext.get().getCurrUserId(), "bos_user");
        for (RelationDTO relationDTO : relationList) {
            if (relationDTO.getMasterTableNumber() == null || relationDTO.getMasterTableNumber().isEmpty()) {
                return CustomApiResult.fail("", "存在主表编码为空的数据");
            }
            if (relationDTO.getNumber() == null || relationDTO.getNumber().isEmpty()) {
                return CustomApiResult.fail("", "存在人区客品编码为空的数据");
            }
            if (relationDTO.getSalesman() == null || relationDTO.getSalesman().isEmpty()) {
                return CustomApiResult.fail("", "存在业务员编码为空的数据,主表编码为:{" + relationDTO.getMasterTableNumber() + "}");
            }
            if (relationDTO.getOffice() == null || relationDTO.getOffice().isEmpty()) {
                return CustomApiResult.fail("", "存在区域编码为空的数据,主表编码为:{" + relationDTO.getMasterTableNumber() + "}");
            }
            if (relationDTO.getCustomer() == null || relationDTO.getCustomer().isEmpty()) {
                return CustomApiResult.fail("", "存在客户编码为空的数据,主表编码为:{" + relationDTO.getMasterTableNumber() + "}");
            }
            if (relationDTO.getStart_date() == null || relationDTO.getStart_date().toString().isEmpty()) {
                return CustomApiResult.fail("", "存在开始日期为空的数据,主表编码为:{" + relationDTO.getMasterTableNumber() + "}");
            }
            if (relationDTO.getEnd_date() == null || relationDTO.getEnd_date().toString().isEmpty()) {
                return CustomApiResult.fail("", "存在结束日期为空的数据,主表编码为:{" + relationDTO.getMasterTableNumber() + "}");
            }
            DynamicObject relation = BusinessDataServiceHelper.loadSingle("yrpg_mkt_sales_area_manag",
                    QFilter.of("yrpg_master_data_number = ?", relationDTO.getNumber()).toArray());
            if (relation != null) {
                relation.set("yrpg_end_date", relationDTO.getEnd_date());
            } else {
                    DynamicObject org = BusinessDataServiceHelper.loadSingle("bos_adminorg",
                        QFilter.of("yrpg_mainid = ?", relationDTO.getOffice()).toArray());
                DynamicObject sales = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                        QFilter.of("number = ?", relationDTO.getSalesman()).toArray());
                DynamicObject account = BusinessDataServiceHelper.loadSingle("yrpg_mkt_user",
                        QFilter.of("number = ?", relationDTO.getAccount_man()).toArray());
                DynamicObject mktCustomer = BusinessDataServiceHelper.loadSingle("yrpg_bd_endcustomer",
                        QFilter.of("number = ?", relationDTO.getCustomer()).toArray());

                DynamicObject mktGoods = relationDTO.getGoods() == null || relationDTO.getGoods().isEmpty() ? null :
                        BusinessDataServiceHelper.loadSingle("yrpg_mkt_goods",
                                QFilter.of("number = ?", relationDTO.getGoods()).toArray());
                if (org == null) {
                    return CustomApiResult.fail("", "未查询到区域编码对应的区域,区域编码:{" + relationDTO.getOffice() + "}");
                }
                if (sales == null) {
                    return CustomApiResult.fail("", "未查询到业务员编码对应的业务员,业务员编码:{" + relationDTO.getSalesman() + "}");
                }
                if (mktCustomer == null) {
                    return CustomApiResult.fail("", "未查询到客户编码对应的客户,客户编码:{" + relationDTO.getCustomer() + "}");
                }
                if (relationDTO.getGoods() != null && !relationDTO.getGoods().isEmpty() && mktGoods == null) {
                    return CustomApiResult.fail("", "未查询到品种编码对应的品种,品种编码:{" + relationDTO.getGoods() + "}");
                }
                DynamicObject mktOffice = BusinessDataServiceHelper.loadSingle("yrpg_mkt_office",
                        QFilter.of("number = ?", relationDTO.getOffice()).toArray());
                if (mktOffice == null) {
                    mktOffice = createOfficeInfo(org, user);
                }
                relation = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_sales_area_manag");
                relation.set("yrpg_master_data_number", relationDTO.getNumber());
                relation.set("yrpg_customer_dimension", "1");
                relation.set("yrpg_goods_dimension", "1");
                relation.set("yrpg_org", org);
                relation.set("yrpg_office_id", mktOffice);
                relation.set("yrpg_salesman_id", sales);
                relation.set("yrpg_account_man_id", account);
                relation.set("yrpg_customer_id", mktCustomer);
                relation.set("yrpg_goods_id", mktGoods);
                relation.set("yrpg_start_date", relationDTO.getStart_date());
                relation.set("yrpg_end_date", relationDTO.getEnd_date());
                relation.set("creator", user);
                relation.set("auditor", user);
                relation.set("auditdate", new Date());
                relation.set("billstatus", "C");
            }
            list.add(relation);
        }
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_sales_area_manag", list.toArray(new DynamicObject[0]), OperateOption.create());
        if (result.isSuccess()) {
            ArrayList<String> salesman = new ArrayList<>();
            for (RelationDTO relationDTO : relationList) {
                salesman.add(relationDTO.getSalesman());
            }
            DynamicObject[] load = BusinessDataServiceHelper.load("yrpg_mkt_user", "id,yrpg_is_sales_person",
                    new QFilter("number", QCP.in, salesman).toArray());
            for (DynamicObject dynamicObject : load) {
                dynamicObject.set("yrpg_is_sales_person", true);
            }
            SaveServiceHelper.save(load);
            return CustomApiResult.success("success");
        } else {
            List<ValidateResult> validateErrors = result.getValidateResult().getValidateErrors();
            if (!validateErrors.isEmpty()) {
                List<OperateErrorInfo> allErrorInfo = validateErrors.get(0).getAllErrorInfo();
                if (!allErrorInfo.isEmpty()) {
                    StringBuilder message = new StringBuilder();
                    for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                        message.append(operateErrorInfo.getMessage());
                    }
                    return CustomApiResult.fail("", message.toString());
                } else {
                    return CustomApiResult.fail("", result.getMessage());
                }
            } else {
                return CustomApiResult.fail("", result.getMessage());
            }
        }
    }

    public static DynamicObject createOfficeInfo(DynamicObject org, DynamicObject user) {
        DynamicObject mktOffice = BusinessDataServiceHelper.newDynamicObject("yrpg_mkt_office");
        mktOffice.set("number", org.getString("yrpg_mainid"));
        mktOffice.set("name", org.getString("name"));
        mktOffice.set("yrpg_office", org);
        mktOffice.set("yrpg_is_self", "0");
        mktOffice.set("yrpg_office_type", "1");
        mktOffice.set("createorg", yzj);
        mktOffice.set("useorg", yzj);
        mktOffice.set("org", yzj);
        mktOffice.set("creator", user);
        mktOffice.set("enable", "1");
        mktOffice.set("status", "C");
        OperationResult result = SaveServiceHelper.saveOperate("yrpg_mkt_office", new DynamicObject[]{mktOffice}, OperateOption.create());
        return BusinessDataServiceHelper.loadSingle(result.getSuccessPkIds().get(0), "yrpg_mkt_office");
    }
}
